-------------------------------------------------------------------------------
--   Projekt   : kniha Cislicove systemy a jazyk VHDL
--   Jm. soub. : tb_fibex.vhd
--   Nazev     : Vypis 3.9
--   Popis     : Priklad testovaciho prostredi (testbenche)
-------------------------------------------------------------------------------
--   Datum      Autor             Revize  Poznamka
--   16.8.2006  Martin Poupa      Rev A   Vytvoreno
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity tb_fibex is
end tb_fibex;

architecture behav of tb_fibex is
   -- deklarace testovane komponenty
   component fibex
      generic (
         sirka : positive := 16
      );
      port (
         clk : in std_logic;
         reset : in std_logic;
         vystup : out std_logic_vector (sirka - 1 downto 0);
         preteceni : out std_logic
      );
   end component;

   -- deklarace pomocnych signalu a konstanty
   constant sirka : positive := 16;
   signal clk, clr, overflow : std_logic;
   signal result : std_logic_vector (sirka - 1 downto 0);
begin
   -- pouziti a pripojeni testovane komponenty
   dut : fibex port map (clk, clr, result, overflow);

   -- proces vytvarejici testovaci prostredi
   process
      type tf_integer is file of integer; -- definice typu soubor (integer)
      type tf_text is file of string; -- definice typu soubor (string)
      -- deklarace vstupniho souboru
      file f_vstup : tf_integer open read_mode is "vstup.bin";
      -- deklarace vystupnich souboru
      file f_vyst_bin : tf_integer open write_mode is "vystup.bin";
      file f_vyst_txt : tf_text open write_mode is "vystup.txt";
      variable test_vysl : integer;
   begin
      -- generovani resetu pro testovany komponent
      clk <= '0';
      clr <= '1';
      wait for 100 ns;
      clr <= '0'; -- konec resetu
      while true loop -- testovaci smycka
         -- generovani jedne periody hodin
         clk <= '0';
         wait for 100 ns;
         clk <= '1';
         wait for 100 ns;

         -- test zda komponenta hlasi preteceni, pokud ano - konec testu
         if (overflow = '1') then
            report "Nastalo preteceni, konec simulace." severity note;
            exit;
         end if;

         -- zapis vysledku, ktery vratila komponenta do souboru
         write (f_vyst_bin, to_integer(unsigned(result)));
         write (f_vyst_txt, integer'image(to_integer(unsigned(result))) & " ");

         -- cteni ocekavaneho vysledku ze vstupniho binarniho souboru
         if not endfile (f_vstup) then
            read (f_vstup, test_vysl);
         else
            report "Dosla data ve vstupnim souboru." severity error;
            exit;
         end if;

         -- vysledku z komponentu na ocekavanou hodnotu
         assert test_vysl = to_integer(unsigned(result))
            -- pokud ma vysledek jinou hodnotu, vypis ocekavane hodnoty
            -- a zastaveni simulace
            report "Vysledek nema ocekavanou hodnotu. Ocekavano: " &
               integer'image(test_vysl)
            severity failure;
      end loop;
      -- zavreni otevrenych souboru
      file_close (f_vstup);
      file_close (f_vyst_bin);
      file_close (f_vyst_txt);
      -- vypis hlaseni, konec simulace
      report "Konec simulace" severity note;
      wait;
   end process;
end behav;
